"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const os_1 = __importDefault(require("os"));
const convict_1 = __importDefault(require("convict"));
const utils_1 = require("@terascope/utils");
const connector_utils_1 = require("./connector-utils");
const schema_1 = __importDefault(require("./schema"));
function validateConfig(cluster, schema, namespaceConfig) {
    try {
        const config = convict_1.default(schema || {});
        config.load(namespaceConfig);
        if (cluster.isMaster) {
            config.validate({
                // IMPORTANT: changing this will break things
                // FIXME
                // false is deprecated and will be removed in ^5.0.0
                // must be warn or strict
                allowed: true,
            });
        }
        return config.getProperties();
    }
    catch (err) {
        throw new utils_1.TSError(err, { reason: 'Error validating configuration' });
    }
}
function extractSchema(fn, sysconfig) {
    if (utils_1.isFunction(fn)) {
        return fn(sysconfig);
    }
    if (utils_1.isPlainObject(fn)) {
        return fn;
    }
    return {};
}
/**
 * @param cluster the nodejs cluster metadata
 * @param config the config object passed to the library terafoundation
 * @param sysconfig unvalidated sysconfig
*/
function validateConfigs(cluster, config, sysconfig) {
    if (!utils_1.isPlainObject(config) || utils_1.isEmpty(config)) {
        throw new Error('Terafoundation requires a valid application configuration');
    }
    if (!utils_1.isPlainObject(sysconfig) || utils_1.isEmpty(sysconfig)) {
        throw new Error('Terafoundation requires a valid system configuration');
    }
    const schema = extractSchema(config.config_schema, sysconfig);
    const result = {};
    if (config.schema_formats) {
        config.schema_formats.forEach((format) => {
            convict_1.default.addFormat(format);
        });
    }
    for (const [namespace, namespaceConfig] of Object.entries(sysconfig)) {
        // terafoundation
        if (namespace === 'terafoundation') {
            result[namespace] = validateConfig(cluster, schema_1.default, namespaceConfig);
            result[namespace].connectors = {};
            const connectors = sysconfig[namespace].connectors || {};
            for (const [connector, connectorConfig] of Object.entries(connectors)) {
                const innerSchema = connector_utils_1.getConnectorSchema(connector);
                result[namespace].connectors[connector] = {};
                for (const [endpoint, endpointConfig] of Object.entries(connectorConfig)) {
                    result[namespace].connectors[connector][endpoint] = validateConfig(cluster, innerSchema, endpointConfig);
                }
            }
        }
        else {
            result[namespace] = validateConfig(cluster, schema[namespace], namespaceConfig);
        }
    }
    // Annotate the config with some information about this instance.
    const hostname = os_1.default.hostname();
    if (process.env.POD_IP) {
        result._nodeName = process.env.POD_IP;
    }
    else if (cluster.worker) {
        result._nodeName = `${hostname}.${cluster.worker.id}`;
    }
    else {
        result._nodeName = hostname;
    }
    return result;
}
exports.default = validateConfigs;
//# sourceMappingURL=validate-configs.js.map